#include <unistd.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <ctype.h>

#include <gnome.h>

#include "napster.h"
#include "gnapster.h"
#include "callbacks.h"
#include "ui.h"
#include "upload.h"
#include "network.h"
#include "commands.h"
#include "download.h"
#include "queue.h"
#include "resume.h"
#include "utils.h"

#include "config.h"

#include "debug.h"

extern ResumeDlg resume_dlg;
extern ConnInfo conn_info;
extern SearchTab search_tab;
extern BrowseDlg browse_dlg;

void modify_resume_entry(char *newpath, Download *download) {
   FILE *f;
   char *hld, *file, *size, *md5, *checksum, *dptr;
   GList *ptr, *felem;
   unsigned long rsize;
   
   f = open_resume("r");
   if (!f)
     return;
   
   felem = read_file(f);
   
   fclose(f);
   
   f = open_resume("w");
   if (!f)
     return;
   
   for(ptr=felem; ptr; ptr=ptr->next) {
      dptr = ptr->data;
      
      file = next_arg_first(dptr, &dptr, &hld);
      size = next_arg(dptr, &dptr);
      md5 = next_arg(dptr, &dptr);
      if (!md5)
	continue;
      
      checksum = next_arg_full(md5, &md5, '-');
      convert(next_arg(md5, &md5), "%lu", &rsize);
      
      if (checksum && rsize > 0) {
	 if (!strcmp(checksum, download->id) && download->total == rsize)
	   fprintf(f, "\"%s\" %i %s-%lu\n", newpath, download->size, checksum,
		   rsize);
	 else
	   fprintf(f, "\"%s\" %s %s-%lu\n", file, size, checksum, rsize);
      }
      
      d_free(hld);
   }
   
   g_list_free(felem);
   
   fclose(f);
}

void remove_resume_entry(Download *download) {
   FILE *f;
   char *hld, *file, *size, *md5, *dptr;
   GList *ptr, *felem;
   char *checksum;
   int rsize;
   
   f = open_resume("r");
   if (!f)
     return;
   
   felem = read_file(f);
   
   fclose(f);
   
   f = open_resume("w");
   if (!f)
     return;
   
   for(ptr=felem; ptr; ptr=ptr->next) {
      dptr = ptr->data;
      
      file = next_arg_first(dptr, &dptr, &hld);
      size = next_arg(dptr, &dptr);
      md5 = next_arg(dptr, &dptr);
      
      checksum = next_arg_full(md5, &md5, '-');
      rsize = my_atoi(next_arg(md5, &md5));
      
      if (rsize > 0 && checksum) {
	 if (!strcmp(checksum, download->id) && download->total == rsize) {
	 } else
	   fprintf(f, "\"%s\" %s %s-%i\n", file, size, checksum, rsize);
      }
      
      d_free(hld);
   }
   
   g_list_free(felem);
   
   fclose(f);
}

void append_resume_entry(Download *download) {
   FILE *f;
   char *hld, *file, *size, *md5, *dptr, *checksum;
   GList *ptr, *felem;
   int rsize;
   
   f = open_resume("r");
   if (f) {
      felem = read_file(f);

      fclose(f);

      for(ptr=felem; ptr; ptr=ptr->next) {
	 dptr = ptr->data;

	 file = next_arg_first(dptr, &dptr, &hld);
	 size = next_arg(dptr, &dptr);
	 md5 = next_arg(dptr, &dptr);
	 if (!md5)
	   continue;

	 checksum = next_arg_full(md5, &md5, '-');
	 rsize = my_atoi(next_arg(md5, &md5));
	 
	 if (!strcmp(checksum, download->id) && rsize == download->total)
	   break;

	 d_free(hld);
      }

      g_list_free(felem);
   }
   
   f = open_resume("a");
   if (!f)
     return;
   
   fprintf(f, "\"%s\" %i %s-%lu\n", download->fpath, download->size,
	     download->id, download->total);
   
   fclose(f);
}

void resume_dlg_clist_cb() {
}

void resume_dlg_cb(GtkWidget *w, gpointer data) {
   char *s;
   
   if (!data)
     return;
   
   s = (char *)data;
   
   if (!strcmp(s, "resume")) {
      /* Resume searches are actually broken server side and on the client-side
       * of the official client...so kind of useless */
      Resume *temp;
      
      temp = gnapster_get_selection_data(resume_dlg.clist, NULL);
      if (!temp)
	return;
      
      napster_send(conn_info.sock, NAPSTER_RESUME_SEARCH, "%s-%i %i",
		   temp->checksum, temp->remotesize, temp->remotesize);
      
      conn_info.searching = 1;
      
      gtk_clist_clear(GTK_CLIST(search_tab.clist));
      gtk_clist_freeze(GTK_CLIST(search_tab.clist));
      gtk_entry_set_editable(GTK_ENTRY(search_tab.gtk_entry), 0);
   } else if (!strcmp(s, "remove")) {
      Resume *temp;
      Download *download;
      
      temp = gnapster_get_selection_data(resume_dlg.clist, NULL);
      if (!temp)
	return;
      
      download = d_new(N_DOWNLOAD);
      
      download->total = temp->remotesize;
      download->id = d_strdup(temp->checksum);
      
      remove_resume_entry(download);
      
      d_free(download);
      
      populate_resume_dlg();
      
      return;
   }
   
   gnome_dialog_close(GNOME_DIALOG(resume_dlg.window));
   resume_dlg.window = NULL;
}

void populate_resume_dlg() {
   FILE *f;
   char *hld, *file, *size, *md5, *checksum, *tf, *dptr;
   GList *ptr, *felem;
   Resume *resume;
   int rsize;
   
   f = open_resume("r");
   if (!f)
     return;
   
   gtk_clist_freeze(GTK_CLIST(resume_dlg.clist));
   gtk_clist_clear(GTK_CLIST(resume_dlg.clist));
   
   felem = read_file(f);
   
   fclose(f);
   
   for(ptr=felem; ptr; ptr=ptr->next) {
      dptr = ptr->data;
      
      file = next_arg_first(dptr, &dptr, &hld);
      size = next_arg(dptr, &dptr);
      md5 = next_arg(dptr, &dptr);
      if (!md5)
	break;
      
      checksum = next_arg_full(md5, &md5, '-');
      rsize = my_atoi(next_arg(md5, &md5));
      
      
      resume = d_new(N_RESUME);

      resume->file = d_strdup(file);
      resume->checksum = d_strdup(checksum);
      resume->remotesize = rsize;
      
      tf = strrchr(resume->file, '/');
      if (tf)
	tf++;
      
      gnapster_clist_append(resume_dlg.clist, NULL, resume, tf, 
			    simplify_size(resume->remotesize), NULL);
      
      d_free(hld);
   }
   
   g_list_free(felem);

   gtk_clist_thaw(GTK_CLIST(resume_dlg.clist));
}

void resume_download_cb() {
   if (resume_dlg.window)
     return;
   
   create_resume_dlg();
   populate_resume_dlg();
}

FILE *open_resume(char *flags) {
   FILE *f;
   char *conf;
   
   conf = d_strdup_printf(DEBUG "%s/.gnapster/resume", getenv("HOME"));
   
   f = fopen(conf, flags);
   
   if (!f)
     perror("fopen (~/.gnapster/resume)");
   
   d_free(conf);
   
   return f;
}
