#include <stdio.h>

#include <unistd.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <ctype.h>

#include <gnome.h>

#include "napster.h"
#include "gnapster.h"
#include "callbacks.h"
#include "ui.h"
#include "upload.h"
#include "network.h"
#include "commands.h"
#include "download.h"
#include "queue.h"
#include "utils.h"

#include "chan.h"
#include "text.h"
#include "xtext.h"
#include "debug.h"


extern ConsoleTab console_tab;
extern GnapsterMain gnapster_main;
extern MOTDTab motd_tab;
extern UserInfo user_info;

char xtext_buf[2048 + 1];
int xtext_pos = 0;

/* Some of this code was taken from XChat with the authors permission, it was 
 * then hacked into Gnapster's system of text display. */

GdkColor colors[] = {
   { 0, 0,      0,      0 },
   { 0, 0xcccc, 0xcccc, 0xcccc },
   { 0, 0,      0,      0xcccc },
   { 0, 0,      0xcccc, 0 },
   { 0, 0xcccc, 0,      0 },
   { 0, 0xbbbb, 0xbbbb, 0 },
   { 0, 0xbbbb, 0,      0xbbbb }, /* 6  purple */
   { 0, 0xffff, 0xaaaa, 0 },      /* 7  orange */
   { 0, 0xffff, 0xffff, 0 },      /* 8  yellow */
   { 0, 0,      0xffff, 0 },      /* 9  green */
   { 0, 0,      0xcccc, 0xcccc }, /* 10 aqua */
   { 0, 0,      0xffff, 0xffff }, /* 11 light aqua */
   { 0, 0,      0,      0xffff }, /* 12 blue */
   { 0, 0xffff, 0,      0xffff }, /* 13 pink */
   { 0, 0x7777, 0x7777, 0x7777 }, /* 14 grey */
   { 0, 0x9999, 0x9999, 0x9999 }, /* 15 light grey */
   { 0, 0,      0,      0xcccc }, /* 16 blue markBack */
   { 0, 0xeeee, 0xeeee, 0xeeee }, /* 17 white markFore */
   { 0, 0xcccc, 0xcccc, 0xcccc }, /* 18 foreground (white) */
   { 0, 0,      0,      0 },
};

GdkFont *load_font() {
   GdkFont *font;
   char *fontname;

   fontname = user_info.default_font;

   font = gdk_font_load(fontname);
   if (!font)
     font = gdk_font_load("fixed");
   
   return font;
}

void generate_colors(GtkWidget *widget) {
   int i;
   
   if (!colors[0].pixel)
     for(i=0; i<20; i++) {
	colors[i].pixel = (unsigned long)((colors[i].red & 0xff00) * 256 +
					  (colors[i].green & 0xff00) +
					  (colors[i].blue & 0xff00) / 256);
	gdk_color_alloc(gtk_widget_get_colormap(widget), &colors[i]);
     }
}

char *text_insert_next_val(char *s, char **next, int *col) {
   char *ptr, *str;
   
   if (!s || !*s)
     return NULL;
   
   ptr = s;
   str = ptr;
   while(*str) {
      if (*str >= 1 && *str <= 2) {
	 *col = *str;
	 break;
      }
      str++;
   }
   
   if (*str)
     *str++ = 0;
   
   if (next)
     *next = str;
   
   return ptr;
}

char *real_text_insert(GtkWidget *text, char *fmt) {
   char *ret = NULL, *fmtptr, *ptr = NULL;
   int col = -1, next_col = 1;
   char buf[1024], b_str[2048];
   
   bzero(buf, sizeof(buf));
   bzero(b_str, sizeof(b_str));
   fmtptr = fmt;
   while(ptr || col == -1) {
      if (col == -1)
	ret = fmt;
      col = next_col;
      ptr = text_insert_next_val(fmtptr, &fmtptr, &next_col);
      if (!ptr)
	break;

      if (col == 2) 
	g_snprintf(buf, sizeof(buf), "\003%i%s", col + 8, ptr);
      else
	g_snprintf(buf, sizeof(buf), "\017%s", ptr);
      if (strlen(buf) + strlen(b_str) < 2048)
	strcat(b_str, buf);
   }
   
   gtk_xtext_append(GTK_XTEXT(text), b_str, strlen(b_str));
   
   return ret;
}

void gnapster_text_insert(int val, char *prefix, char *fmt, ...) {
   GtkWidget *text, *vbox, *parent;
   char *str;
   char *fmtptr, *format_str;
   int i;
   va_list args;
   
   if (val == CURR)
     val = gnapster_get_pagenum();
   else if (val < 0)
     return;
   
   va_start(args, fmt);
   format_str = d_strdup_vprintf(fmt, args);
   va_end(args);
   
   if (!format_str)
     return;
   
   parent = console_tab.notebook;
   switch(val) {
    case CONSOLE:
      text = console_tab.text;
      vbox = console_tab.vbox;
      break;
    case MOTD:
      parent = gnapster_main.notebook;
      text = motd_tab.text;
      vbox = motd_tab.vbox;
      break;
    default:
	{
	   ChannelInfo *chan;
	   
	   chan = g_list_nth_data(console_tab.channel_list, (val - 2));
	   if (!chan)
	     return;
	   
	   text = chan->text;
	   vbox = chan->vbox;
	}
      break;
   }
   
   if (strcmp(prefix, "")) {
//     gtk_xtext_append(GTK_XTEXT(text), prefix, strlen(prefix));
//      char *npfx = d_strdup(prefix);
      
//      real_text_insert(text, npfx);
//      d_free(npfx);
      str = d_strdup_printf(DEBUG "%s%s", prefix, format_str);
      d_free(format_str);
      format_str = str;
   }

   for(i=0;format_str[i];i++) {
      if (format_str[i] == '\n') {
	 xtext_buf[xtext_pos] = 0;
	 xtext_pos = 0;
	 
	 fmtptr = real_text_insert(text, xtext_buf);
      } else
	xtext_buf[xtext_pos++] = format_str[i];
   }
   
//   fmtptr = real_text_insert(text, format_str);
//   d_free(fmtptr);
   d_free(format_str);
   
   if (xtext_pos)
     return;
//   gtk_xtext_append(GTK_XTEXT(text), format_str, strlen(format_str));
   
   if (parent == console_tab.notebook)
     gnapster_highlight(gnapster_main.notebook, console_tab.main_vbox);
   gnapster_highlight(parent, vbox);
}
