/* upload.c - uhh, upload/share stuff :) */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>

#include <dirent.h>

#include <time.h>

#include <gnome.h>

#include "napster.h"
#include "gnapster.h"
#include "ui.h"
#include "callbacks.h"
#include "text.h"
#include "utils.h"

#include "upload.h"
#include "md5.h"

#include "network.h"

#include "debug.h"

#ifndef MAP_FAILED
#define MAP_FAILED (void *) -1
#endif

extern ConnInfo conn_info;
extern UserInfo user_info;
extern UploadTab upload_tab;
extern BuildDlg build_dlg;

extern int building;

extern GList *total_list;

char *get_checksum(int fd, unsigned long mapsize) {
   unsigned char digest[16];
   md5_state_t state;
   char buffer[2048 + 1];
   struct stat st;
   unsigned long size = 3000000;
   int di = 0;
   int rc;
   char *m;
   
   *buffer = 0;
   md5_init(&state);
   if ((fstat(fd, &st)) == -1) 
     return d_strdup("");
   
   if (!mapsize) {
      if (st.st_size < size)
	size = st.st_size;
   } else if (st.st_size < mapsize)
     size = st.st_size;
   else 
     size = mapsize;
   
   if ((m = mmap((void *)0, size, PROT_READ, MAP_PRIVATE, fd, 0)) != MAP_FAILED) {
      md5_append(&state, (unsigned char *)m, size);
      md5_finish(digest, &state);
      munmap(m, size);
      memset(buffer, 0, 200);
      for(di=0, rc=0; di<16; ++di, rc+=2)
	g_snprintf(&buffer[rc], 2048, "%02x", digest[di]);
      strcat(buffer, "-");
      sprintf(buffer, "%s%li", buffer, st.st_size);
   }
   
   return d_strdup(buffer);
}

char *conv_win(char *s) {
   char *ptr;
   
   ptr = s;
   if (!ptr || !*ptr) return NULL;
   
   while(*ptr) {
      if (*ptr == '/') *ptr = '\\';
      ptr++;
   }
   
   return s;
}

char *conv_unix(char *s) {
   char *ptr;
   
   ptr = s;
   if (!ptr || !*ptr) return NULL;
   
   while(*ptr) {
      if (*ptr == '\\') *ptr = '/';
      ptr++;
   }
   
   return s;
}

GList *list_match(GList *list, char *file) {
   MP3Stats *mp3;
   GList *ptr;

   if (!list) return NULL;
   if (!file) return NULL;
   
   ptr = total_list;
   while(ptr) {
      mp3 = ptr->data;
      if (!mp3)
	break;
      
      if (!strcmp(conv_unix(mp3->filename), file))
	return ptr;
      
      ptr = ptr->next;
   }
   
   return NULL;
}

int check_ext(char *p) {
   char *ext;
   
   ext = strrchr(p, '.');
   if (!ext) 
     return 0;
   
   if (strcasecmp(ext, ".mp3"))
     return 0;
   
   return 1;
}

int read_byte(int fd, unsigned long *hdr) {
   unsigned char c;
   int n;
   
   *hdr <<= 8;
   
   n = read(fd, &c, 1);
   if (n <= 0)
     return n;

   *hdr |= c;

   return n;
}

int find_header(int fd, unsigned long *hdr) {
   int offs, n;
   
   offs = 0;
   
   while((n = read_byte(fd, hdr))) {
      if (((*hdr >> 16) & 0xffe0) == 0xffe0)
	return (offs - 3);
      offs += n;
   }
   
   return -1;
}

int get_bits(unsigned long *hdr, int bits) {
   int x;
   
   x = (*hdr >> (32 - bits));
   
   *hdr <<= bits;
   
   return x;
}

void get_header(unsigned long hdr, MP3Header **h) {
   MP3Header *hd;
   
   hd = *h;
   
   get_bits(&hdr, 11); /* skip the frame sync, we already know what this is */
   hd->index = get_bits(&hdr, 1);
   hd->id = get_bits(&hdr, 1);
   hd->layer = get_bits(&hdr, 2);
   hd->protection_bit = get_bits(&hdr, 1);
   hd->bitrate_index = get_bits(&hdr, 4);
   hd->frequency = get_bits(&hdr, 2);
   hd->padding_bit = get_bits(&hdr, 1);
   hd->private_bit = get_bits(&hdr, 1);
   hd->mode = get_bits(&hdr, 2);
   hd->mode_ext = get_bits(&hdr, 2);
   if (!hd->mode)
     hd->mode_ext = 0;
   hd->copyright = get_bits(&hdr, 1);
   hd->original = get_bits(&hdr, 1);
   hd->emphasis = get_bits(&hdr, 2);   
}

int process_header(int fd, unsigned long hdr, MP3Stats **stats) {
   MP3Stats *s;
   MP3Header *h;
   struct stat st;
   unsigned short bitrates[2][3][15] = {
      { 
	 { 0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256 },
	 { 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160 },
	 { 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160 }
      },
      {
	 { 0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448 },
	 { 0, 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384 },
	 { 0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320 }
      }
   };
   unsigned short frequencies[2][2][3] = {
      {
	 { 11025, 12000, 8000 },
	 { 0, 0, 0 },
      },
      {
	 { 22050, 24000, 16000 },
	 { 44100, 48000, 32000 },
      }
   };
   
   s = *stats;
   
   h = malloc(sizeof(MP3Header));
   
   get_header(hdr, &h);
   
   if (h->id > 1 || h->layer > 2 || h->bitrate_index > 14)
     return -1;
   
   fstat(fd, &st);
   
   s->filesize = st.st_size;
   s->mtime = st.st_mtime;
   
   s->bitrate = bitrates[h->id][3 - h->layer][h->bitrate_index];
   s->frequency = frequencies[h->index][h->id][h->frequency];
   
   if (s->frequency > 0) {
      int framesize, total;
      
      framesize = (((h->id ? 144000 : 72000) * s->bitrate) / s->frequency);
      total = (s->filesize / (framesize + 1)) - 1;
      s->time = (time_t) (total * (h->id ? 1152 : 576) / s->frequency);
   } else
     s->time = 0;
   
   free(h);
   
   return 1;
}

int process_mp3(int fd, MP3Stats **stats) {
   int offs;
   unsigned long hdr;
   
   hdr = 0;
   offs = find_header(fd, &hdr);
   if (offs < 0)
     return offs;
   
   if (process_header(fd, hdr, stats) < 0)
     return -1;
   
   return offs;
}

void process_file(char *p, struct stat st, int pd, FILE *fd, int flag, GList *list) {
   int f;
   GList *match = NULL;
   MP3Stats *mp3, *match_mp3 = NULL;
   char *buffer;
   int build = 1;

   if (!p) 
     return;
   
   if (!check_ext(p)) 
     return;
   
   if (flag == UPDATE_LIST) {
      match = list_match(list, p);
      if (match) {
	 match_mp3 = (MP3Stats *)match->data;
	 if (!match_mp3) 
	   return;

	 if (st.st_mtime <= match_mp3->mtime)
	   build = 0;
      }
   }
   if (build) {
      int offs;
      
      write(pd, p, strlen(p));

      f = open(p, O_RDONLY);
      if (f < 0) {
	 fprintf(stderr, "could not open %s\n",
		 p);
	 return;
      }
      
      mp3 = d_malloc(sizeof(MP3Stats));
      memset(mp3, 0, sizeof(MP3Stats));
      
      if ((offs = process_mp3(f, &mp3)) < 0) {
	 fprintf(stderr, "could not process mp3 %s\n",
		 p);
	 mp3_free(mp3);
	 return;
      }
      
      lseek(f, offs, SEEK_SET);

      mp3->filename = d_strdup(p);
      mp3->checksum = get_checksum(f, 300000);

      close(f);
      
      buffer = d_strdup_printf(DEBUG "\"%s\" %s %li %i %i %lu",
			       conv_win(mp3->filename), mp3->checksum, 
			       mp3->filesize, mp3->bitrate, mp3->frequency, 
			       mp3->time);
      fprintf(fd, "%lu %s\n", mp3->mtime, buffer);
	 
      if (conn_info.connected)
	napster_send(conn_info.sock, NAPSTER_SUBMIT, buffer);

      d_free(buffer);

      if (!building) 
	return;

      mp3_free(mp3);
   } else {
      if (!match_mp3) 
	return;

      /* Not rebuilding data, simply writing what we have... */
      fprintf(fd, "%li \"%s\" %s %li %i %i %lu\n", match_mp3->mtime,
	      conv_win(match_mp3->filename), match_mp3->checksum, 
	      match_mp3->filesize, match_mp3->bitrate, match_mp3->frequency, 
	      match_mp3->time);
   }
   if (match) {
      GList *link;
      
      link = g_list_find(total_list, match->data);
      total_list = g_list_remove_link(total_list, link);
      d_free(match_mp3->filename);
      d_free(match_mp3->checksum);
      d_free(match_mp3);
   }
}

void rodir(int pd, FILE *fd, char *n, int flags, GList *list) {
   DIR *dir;
   char p[1024 + 1];
   char link[1024 + 1];
   struct dirent *dptr;
   struct stat st;
   struct stat lst;

   dir = opendir(n);
   if (!dir) 
     return;
   
   for(;;) {
      dptr = readdir(dir);
      if (dptr == NULL) 
	break;
      
      g_snprintf(p, sizeof(p), "%s/%s", n, dptr->d_name);
      stat(p, &st);
      
      if (strcmp(dptr->d_name, ".") && strcmp(dptr->d_name, "..")) {
	 if ((S_ISDIR(st.st_mode)) && (!S_ISLNK(st.st_mode)))
	   rodir(pd, fd, p, flags, list);
	 else if (S_ISLNK(st.st_mode)) {
	    memset(&link, 0, sizeof(link));

	    if (readlink(p, link, 1024) != -1) {
	       if (stat(link, &lst) != -1) {
		  if (S_ISDIR(st.st_mode))
		    rodir(pd, fd, link, flags, list);
		  else
		    process_file(link, st, pd, fd, flags, list);
	       }
	    }
	 } else {
	    if (!building)
	      return;
	    
	    process_file(p, st, pd, fd, flags, list);
	 }
      }
   }
}

void build_pipe_stop(int sig) {
   building = 0;
}

void build_pipe_read(gpointer data, int source, GdkInputCondition condition) {
   char buf[2048];
   char *file;
   int n;
   
   n = read(source, buf, 2048 - 1);
   if (n <= 0) {
      if (building)
	build_dlg_cancel_cb();
   }
   
   buf[n] = '\0';
   file = strrchr(buf, '/');
   if (!file) {
      if (!strcmp(buf, "BUILD_DONE")) {
	 build_dlg.cpid = 0;
	 build_dlg_cancel_cb();
      }
      return;
   } else if (!strcmp(file, "/BUILD_DONE")) {
      build_dlg.cpid = 0;
      build_dlg_cancel_cb();
      return;
   }
   file++;
   gtk_label_set_text(GTK_LABEL(build_dlg.label), file);
}

void setup_build_dlg(FILE *f, char *path, int flags, GList *list) {
   int pd[2], cpid;
   
   building = 1;
   pipe(pd);
   if ((cpid = fork()) == 0) {
      close(pd[0]);
      signal(SIGUSR1, build_pipe_stop);
      rodir(pd[1], f, path, flags, list);
      write(pd[1], "/BUILD_DONE", strlen("/BUILD_DONE"));
      fclose(f);
      _exit(0);
   }
   fclose(f);   
   build_dlg.cpid = cpid;
   create_build_dlg();
   build_dlg.pipe_input = gdk_input_add(pd[0], GDK_INPUT_READ,
					build_pipe_read, NULL);
   gnome_dialog_run_and_close(GNOME_DIALOG(build_dlg.window));
   wait(NULL);
}

void remove_slash(char *s) {
   char *ptr;
   
   if (!s) return;
   ptr = strrchr(s, '/');
   if (!ptr) 
     return;
   if (strlen(ptr) == 1) 
     ptr[0] = 0;
}

FILE *open_shared(char *flags) {
   FILE *f;
   char *conf;
   
   conf = d_strdup_printf(DEBUG "%s/.gnapster/shared", getenv("HOME"));
   
   f = fopen(conf, flags);
   
   if (!f)
     perror("fopen (~/.gnapster/shared)");
   
   d_free(conf);
   
   return f;
}

int parse_shared_entry(char *s, time_t *mtime, char **file,
		       char **md5, unsigned long int *size, 
		       long *bitrate, unsigned long int *freq,
		       time_t *time) {
   char *mtime_str, *size_str, *bitrate_str, *freq_str, *time_str;

   mtime_str = next_arg(s, &s);
   *file = next_arg(s, &s);
   *md5 = next_arg(s, &s);
   size_str = next_arg(s, &s);
   bitrate_str = next_arg(s, &s);
   freq_str = next_arg(s, &s);
   time_str = next_arg(s, &s);
   
   if (!time_str) return -1;
   
   sscanf(mtime_str, "%li", mtime);
   sscanf(size_str, "%lu", size);
   sscanf(bitrate_str, "%li", bitrate);
   sscanf(freq_str, "%lu", freq);
   sscanf(time_str, "%li", time);
   
   return 1;
}

int shared_list_handle(char *path, int flags) {
//   GList *total_files = NULL;
   FILE *f;
   int ret = 0;
   
   if (flags == SUBMIT_LIST)
     if (!conn_info.connected || conn_info.connecting)
       return 0;
   
   if (!path) 
     return 0;
   if (!strlen(path))
     return 0;
   
   remove_slash(path);
   
   if (flags != UPDATE_LIST)
     user_info.sharing = user_info.share_size = 0;
   
   if (flags != BUILD_LIST) {
      char buf[1024];
      unsigned long int size, freq;
      char *file = NULL, *md5 = NULL;
      long int bitrate;
      time_t mtime, time;
      MP3Stats *mp3;
      
      f = open_shared("r");
      if (!f) return 0;
      
      /* Don't let the list already exist at this point */
      total_list = mp3_list_free(total_list);
      while(!feof(f)) {
	 fgets(buf, sizeof(buf) - 1, f);
	 if (feof(f)) break;
	 if (parse_shared_entry(buf, &mtime, &file, &md5, &size, &bitrate, &freq, &time) < 0)
	   break;
	 
	 if (flags == SUBMIT_LIST) {
	    user_info.sharing++;
	    user_info.share_size += size;
	 }
	 
	 mp3 = d_calloc(sizeof(MP3Stats));
	 mp3->filename = d_strdup(file);
	 mp3->checksum = d_strdup(md5);
	 mp3->filesize = size;
	 mp3->bitrate = bitrate;
	 mp3->frequency = freq;
	 mp3->time = time;
	 mp3->mtime = mtime;

	 total_list = g_list_append(total_list, mp3);
      }
      fclose(f);
      
      if (flags == SUBMIT_LIST) {
	 char *temp_str;
	 temp_str = d_strdup_printf(DEBUG "Sharing %i (%.02f gigabytes)", user_info.sharing,
				    (((float)user_info.share_size / (float)1024) / (float)1024) / (float)1024);
	 gtk_label_set_text(GTK_LABEL(upload_tab.label), temp_str);
	 d_free(temp_str);
	 
	 gnapster_text_insert(CONSOLE, SYSTEM, "Submitting mp3 list...\n");
	 
	 conn_info.upload_input = gdk_input_add(conn_info.sock, GDK_INPUT_WRITE,
						network_handle_submit, total_list);
	 
	 ret = 1;
      }
   } else if (file_exists(path)) {
      f = open_shared("w");
      if (!f) 
	return 0;
      
      setup_build_dlg(f, path, flags, NULL);
   }
   if (total_list && flags == UPDATE_LIST) {
      GList *ptr;
      
      f = open_shared("w");
      if (!f) return 0;
      
      setup_build_dlg(f, path, flags, total_list);
      
      ptr = total_list;
      while(ptr) {
	 if (ptr->data) {
	    MP3Stats *mp3ptr;
	    
	    mp3ptr = (MP3Stats *)ptr->data;
	    d_free(mp3ptr->filename);
	    d_free(mp3ptr->checksum);
	    d_free(mp3ptr);
	    mp3ptr = NULL;
	 }
	 ptr = ptr->next;
      }
      g_list_free(total_list);
      total_list = NULL;
   }
   
   return ret;
}
